/*
 * check_eids() checks to see if the euid and egid are what we want
 */
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <syslog.h>
#include <errno.h>

#include "lp.local.h"

uid_t uid, euid;

int
check_eids(char *title, uid_t euid, gid_t egid)
{ 
    uid_t ckeuid = geteuid();
    gid_t ckegid = getegid();

    if ( ((euid != (uid_t)EOF) && (ckeuid != euid))
      || ((egid != (gid_t)EOF) && (ckegid != egid)) ) {
	fprintf(stderr, "%s: configuration problem!\n"
			"%*s - euid is %u, should be %u\n"
			"%*s - egid is %u, should be %u\n",
			title,
			strlen(title), "", ckeuid, euid,
			strlen(title), "", ckegid, egid);
	return -1;
    }
    uid = getuid();
    euid = geteuid();
    return 0;
} /* check_eids */


/*
 * wanted() prints out a set of permissions in human-readable form
 */
static char *
wanted(unsigned int permissions, unsigned int mask)
{
    static char bfr[80];
    char *p=bfr;


    if ( (mask & (S_IRWXU|S_ISUID) ) && (permissions & (S_IRWXU|S_ISUID)) ) {
	*p++ = 'u';
	*p++ = '=';
	if (permissions & S_IRUSR)
	    *p++ = 'r';
	if (permissions & S_IWUSR)
	    *p++ = 'w';
	if (permissions & (S_IXUSR|S_ISUID))
	    *p++ = (permissions & S_ISUID) ? 's' : 'x';
    }
    if ( (mask & (S_IRWXG|S_ISGID)) && (permissions & (S_IRWXG|S_ISGID)) ) {
	if (p != bfr)
	    *p++ = ',';
	*p++ = 'g';
	*p++ = '=';
	if (permissions & S_IRGRP)
	    *p++ = 'r';
	if (permissions & S_IWGRP)
	    *p++ = 'w';
	if (permissions & (S_IXGRP|S_ISGID))
	    *p++ = (permissions & S_ISGID) ? 's' : 'x';
    }
    if ( (mask & S_IRWXO) && (permissions & S_IRWXO) ) {
	if (p != bfr)
	    *p++ = ',';
	*p++ = 'o';
	*p++ = '=';
	if (permissions & S_IROTH)
	    *p++ = 'r';
	if (permissions & S_IWOTH)
	    *p++ = 'w';
	if (permissions & S_IXOTH)
	    *p++ = 'x';
    }
    if (p == bfr)
	return "none";

    *p = 0;
    return bfr;
} /* wanted */


/*
 * check_spooldir() checks out a spooldir to see if it's owned by the
 * proper user and group, and that it has the proper permissions.
 */
int
check_spooldir(char *dir, unsigned int permissions, unsigned int mask)
{
    struct stat st;

    if (stat(dir, &st) != 0) {
	fprintf(stderr, "%s: %s", dir, strerror(errno));
	return EOF;
    }

    if (mask == 0)
	mask = permissions;

    if (st.st_uid != LPUID || st.st_gid != LPGID) {
	fprintf(stderr, "%s: ownerships should be user %u, group %u.\n",
			    dir, LPUID, LPGID);
	return EOF;
    }

    /* mask off mode bits we don't care about */
    st.st_mode &= MODEBITS & mask;

    if ( (st.st_mode & permissions) != permissions) {
	fprintf(stderr, "%s: permissions are bad (should be %s)\n",
				    dir, wanted(permissions, mask));
	return EOF;
    }
    return 0;
} /* check_spooldir */
