/*
 *   Copyright (c) 2002 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _CURSE_DEFINED
#define _CURSE_DEFINED	1

#include "../config.h"

#if WITH_NCURSES
#   include <ncurses/curses.h>
#   ifdef HAVE_PANEL
#      include <ncurses/panel.h>
#   endif
#   ifdef KEY_MOUSE
#       define VERMIN	1
#   else
#       define VERMIN	0
#	define MEVENT	void
#   endif

#   define READONLY_COLOR	A_DIM
#   define CURRENT_COLOR	A_BOLD

#   define NT_RARROW		ACS_RARROW
#   define NT_LARROW		ACS_LARROW

#   define setcolor(w,c)	wattrset(w,c)

#else
#   include <ncurses/curses.h>
    typedef int chtype;
#   define VERMIN	0
#   define MEVENT	void

#   define READONLY_COLOR	0
#   define CURRENT_COLOR	0x01

#   define NT_RARROW		'>'
#   define NT_LARROW		'<'

extern void setcolor(WINDOW *w, int c);
#endif

extern int ndwgetch(WINDOW *w);

#if !HAVE_WADDNSTR
extern void waddnstr(WINDOW *, char *, int);
#endif

#if !HAVE_BEEP
extern void beep();
#endif

#if !HAVE_PANEL
extern void ndredraw();
#endif

#if !HAVE_KEYPAD
extern int keypad(void*,int);
#   include "keypad.h"
#endif

#endif /*_CURSE_DEFINED*/
