/*	$OpenBSD: kill.c,v 1.3 1997/02/06 13:29:08 deraadt Exp $	*/
/*	$NetBSD: kill.c,v 1.11 1995/09/07 06:30:27 jtc Exp $	*/

/*
 * Copyright (c) 1988, 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright (c) 1988, 1993, 1994\n\
	The Regents of the University of California.  All rights reserved.\n";
#endif /* not lint */

#ifndef lint
#if 0
static char sccsid[] = "@(#)kill.c	8.4 (Berkeley) 4/28/95";
#else
static char rcsid[] = "$OpenBSD: kill.c,v 1.3 1997/02/06 13:29:08 deraadt Exp $";
#endif
#endif /* not lint */

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef LINUX
extern char *sys_signame[];
#endif

void nosig __P((char *));
void printsignals __P((FILE *, int));
int signame_to_signum __P((char *));
void usage __P((void));
int picksig __P((char*));

char *__progname;

int
main(argc, argv)
	int argc;
	char *argv[];
{
    int errors, numsig, pid;
    int opt;
    int lflag = 0,
	vflag = 0;
    char *ep;

    __progname = basename(argv[0]);

    numsig = 0;

    ++argv, --argc;
    while (argc > 0 && **argv == '-') {
	if (strcmp(*argv, "-l") == 0)
	    lflag = 1;
	else if (strcmp(*argv, "-L") == 0)
	    lflag = vflag = 1;
	else if (strcmp(*argv, "-v") == 0)
	    vflag = 1;
	else if (strncmp(*argv, "-s", 2) == 0) {
	    if (argv[0][2]) {
		numsig = picksig(&argv[0][2]);
	    }
	    else {
		argc--, argv++;
		numsig = picksig(*argv);
	    }
	    if (numsig == EOF)
		usage();
	}
	else {
	    if ((numsig = picksig(&argv[0][1])) == EOF)
		usage();
	}
	argc--, argv++;
    }

    if (lflag) {
	switch (argc) {
	case 0:
	    printsignals(stdout, vflag);
	    exit(0);
	case 1:
	    numsig = picksig(*argv);
	    if (numsig >= 128)
		numsig -= 128;
	    printf("%s\n", sys_signame[numsig]);
	    exit(0);
	default:
	    usage();
	}
    }

    if (argc == 0)
	    usage();

    if (numsig == 0)
	numsig = SIGTERM;
    for (errors = 0; argc; argc--, argv++) {
	pid = strtol(*argv, &ep, 10);
	if (!**argv || *ep) {
	    warnx("illegal process id: %s", *argv);
	    errors = 1;
	}
	else if (kill(pid, numsig) == -1) {
	    warn("%s", *argv);
	    errors = 1;
	}
    }

    exit(errors);
}

int
signame_to_signum(sig)
	char *sig;
{
	int n;

	if (!strncasecmp(sig, "sig", 3))
		sig += 3;
	for (n = 1; n < NSIG; n++) {
		if (!strcasecmp(sys_signame[n], sig))
			return (n);
	}
	return (-1);
}

void
nosig(name)
	char *name;
{

	warnx("unknown signal %s; valid signals:", name);
	printsignals(stderr, 0);
	exit(1);
}

void
printsignals(fp, verbose)
	FILE *fp;
	int verbose;
{
    int n, x;
    int width = 0;

    if (verbose) {
	for (n = 1; n < NSIG; n++) {
	    if (sys_signame[n] == 0)
		continue;
	    x = strlen(sys_signame[n]);
	    if (x > width) width = x;
	}
    }

    for (x = 0, n = 1; n < NSIG; n++) {

	if (sys_signame[n] == 0)
	    continue;

	if (x >= 70) {
	    putchar('\n');
	    x = 0;
	}
	else if (n > 1) {
	    x++;
	    putchar(' ');
	}

	if (verbose) {
	    x += width+5;
	    fprintf(fp, "(%2d)=%-*s", n, width, sys_signame[n]);
	}
	else {
	    x += strlen(sys_signame[n]);
	    fprintf(fp, "%s", sys_signame[n]);
	}
    }
    if (x > 0)
	putchar('\n');
}

void
usage()
{

	(void)fprintf(stderr, "usage: kill [-s signal_name] pid ...\n");
	(void)fprintf(stderr, "       kill -l [exit_status]\n");
	(void)fprintf(stderr, "       kill -signal_name pid ...\n");
	(void)fprintf(stderr, "       kill -signal_number pid ...\n");
	exit(1);
}

int
picksig(char *string)
{
    char *ep;
    int signo;

    if (isdigit(*string)) {
	signo = strtol(string, &ep, 10);

	if (*ep)
	    errx(1, "illegal signal number: %s", string);
    }
    else if ((signo = signame_to_signum(string)) < 0)
	errx(1, "illegal signal name: %s", string);
    return signo;
} /* picksig */
