/*
 * Copyright (c) 1989, 1993, 1994
 *      The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Michael Fischbein.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: print.c,v 1.4.4.1 1995/08/28 10:30:27 davidg Exp $
 */

#ifndef lint
static char sccsid[] = "@(#)print.c	8.4 (Berkeley) 4/17/94";
#endif				/* not lint */

#include <sys/param.h>
#include <sys/stat.h>

#include <err.h>
#include <errno.h>
#include <fts.h>
#include <grp.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <utmp.h>

#include "ls.h"
#include "extern.h"

#ifdef LINUX
#include <linux/kdev_t.h>
#define major(x)  MAJOR(x)
#define minor(x)  MINOR(x)
#endif

static int printaname __P((FTSENT *, u_long, u_long));
static void printlink __P((FTSENT *));
static void printtime __P((time_t));
static int printtype __P((u_int));

#define	IS_NOPRINT(p)	((p)->fts_number == NO_PRINT)

void printscol(dp)
DISPLAY *dp;
{
    FTSENT *p;

    for (p = dp->list; p; p = p->fts_link) {
	if (IS_NOPRINT(p))
	    continue;
	printaname(p, dp->s_inode, dp->s_block);
	putchar('\n');
    }
}

void printlong(dp)
DISPLAY *dp;
{
    struct stat *sp;
    FTSENT *p;
    NAMES *np;
    char buf[20];

    if (dp->list->fts_level != FTS_ROOTLEVEL && (f_longform || f_size))
	printf("total %lu\n", howmany(dp->btotal, blocksize));

    for (p = dp->list; p; p = p->fts_link) {
	if (IS_NOPRINT(p))
	    continue;
	sp = p->fts_statp;
	if (f_inode)
	    printf("%*lu ", dp->s_inode, sp->st_ino);
	if (f_size)
	    printf("%*ld ",
		   dp->s_block, howmany(sp->st_blocks, blocksize));
	strmode(sp->st_mode, buf);
	np = p->fts_pointer;
	printf("%s %*u %-*s  ", buf, dp->s_nlink,
	       sp->st_nlink, dp->s_user, np->user);
	if (!f_nogroup)
	    printf("%-*s  ", dp->s_group, np->group);
	if (f_flags)
	    printf("%-*s ", dp->s_flags, np->flags);
	if (S_ISCHR(sp->st_mode) || S_ISBLK(sp->st_mode))
	    printf("%3d, %3d ",
		   major(sp->st_rdev), minor(sp->st_rdev));
	else if (dp->bcfile)
	    printf("%*s%*ld ",
		   8 - dp->s_size, "", dp->s_size, sp->st_size);
	else
	    printf("%*ld ",
		   dp->s_size, sp->st_size);
	if (f_accesstime)
	    printtime(sp->st_atime);
	else if (f_statustime)
	    printtime(sp->st_ctime);
	else
	    printtime(sp->st_mtime);
	if (f_bracket)
	    printtypeprefix(sp->st_mode);
	printf("%s", p->fts_name);
	if (f_type || f_bracket)
	    printtype(sp->st_mode);
	if (S_ISLNK(sp->st_mode))
	    printlink(p);
	putchar('\n');
    }
}

#define	TAB	8

void printcol(dp)
DISPLAY *dp;
{
    extern int termwidth;
    static FTSENT **array;
    static int lastentries = -1;
    FTSENT *p;
    int base, chcnt, cnt, col, colwidth, num;
    int endcol, numcols, numrows, row;

    /*
     * Have to do random access in the linked list -- build a table
     * of pointers.
     */
    if (dp->entries > lastentries) {
	lastentries = dp->entries;
	if ((array =
	     realloc(array, dp->entries * sizeof(FTSENT *))) == NULL) {
	    warn(NULL);
	    printscol(dp);
	}
    }
    for (p = dp->list, num = 0; p; p = p->fts_link)
	if (p->fts_number != NO_PRINT)
	    array[num++] = p;

    colwidth = 1 + dp->maxlen;
    if (f_inode)
	colwidth += dp->s_inode + 1;
    if (f_size)
	colwidth += dp->s_block + 1;
    if (f_bracket)
	colwidth += 1;
    if (f_type)
	colwidth += 1;

    if (termwidth < 2 * colwidth) {
	printscol(dp);
	return;
    }
    numcols = termwidth / colwidth;

#if 1
    /*
     * Gross format hacking follows: We don't want to leave widows (1 item
     * on a line) in our output, so we reduce the number of columns as needed
     * to make sure that the last column has more than one entry on it. After
     * we've finished with that, we might have a situation where numrows times
     * numcols is > than the number of entries;  if so, shrink the number of
     * columns again. After all this, we'd like to space the columns as evenly
     * across the page as possible, so we pad out the column width until it
     * comes as close as possible to filling the entire width of the page.
     *
     * By now, we're certifiably insane, so we're checked into the county
     * home and ls is left to print out the directory entries.
     */
    while (1) {
	numrows = num / numcols;
	if (num % numcols)
	    ++numrows;
	if (numcols == 1 || (num % numrows) != 1)
	    break;
	--numcols;
    }
    if (numcols < 1)
	numcols = 1;

    if ((numcols-1)*numrows > num)
	numcols--;

    colwidth = termwidth/numcols;

#if 0
printf("termwidth=%d\n"
       "colwidth =%d\n"
       "numcols  =%d\n"
       "compute  =%d\n"
       "slack    =%d\n",
       termwidth, colwidth, numcols,
       numcols*colwidth, termwidth-(numcols*colwidth));
#endif

    while (numcols*colwidth <= termwidth-numcols)
	colwidth++;
#else
    numrows = num / numcols;
    if (num % numcols)
	++numrows;

    while (numcols*colwidth <= termwidth-numcols)
	colwidth++;
#endif

    if (dp->list->fts_level != FTS_ROOTLEVEL && (f_longform || f_size))
	printf("total %lu\n", howmany(dp->btotal, blocksize));
    for (row = 0; row < numrows; ++row) {
	endcol = colwidth;
	for (base = row, chcnt = col = 0; col < numcols; ++col) {
	    chcnt += printaname(array[base], dp->s_inode,
				dp->s_block);
	    if ((base += numrows) >= num)
		break;
	    while (chcnt%colwidth) {
		putchar(' ');
		chcnt++;
	    }
	    endcol += colwidth;
	}
	putchar('\n');
    }
}

/*
 * print [inode] [size] name
 * return # of characters printed, no trailing characters.
 */
static int printaname(p, inodefield, sizefield)
FTSENT *p;
u_long sizefield, inodefield;
{
    struct stat *sp;
    int chcnt;

    sp = p->fts_statp;
    chcnt = 0;
    if (f_inode)
	chcnt += printf("%*lu ", (int) inodefield, sp->st_ino);
    if (f_size)
	chcnt += printf("%*ld ",
		     (int) sizefield, howmany(sp->st_blocks, blocksize));
    if (f_bracket)
	chcnt += printtypeprefix(sp->st_mode);
    chcnt += printf("%s", p->fts_name);
    if (f_type || f_bracket)
	chcnt += printtype(sp->st_mode);
    return (chcnt);
}

static void printtime(ftime)
time_t ftime;
{
    int i;
    char longstring[80];

    strftime(longstring, sizeof(longstring), "%c", localtime(&ftime));
    for (i = 4; i < 11; ++i)
	putchar(longstring[i]);

#define	SIXMONTHS	((365 / 2) * 86400)
    if (f_sectime)
	for (i = 11; i < 24; i++)
	    putchar(longstring[i]);
    else if (time(NULL) >= ftime && ftime + SIXMONTHS > time(NULL))
	for (i = 11; i < 16; ++i)
	    putchar(longstring[i]);
    else {
	putchar(' ');
	for (i = 20; i < 24; ++i)
	    putchar(longstring[i]);
    }
    putchar(' ');
}

static int printtypeprefix(mode)
u_int mode;
{
    switch (mode & S_IFMT) {
    case S_IFDIR:
	putchar('[');
	return 1;
    default:
	putchar(' ');
	return 1;
    }
}

static int printtype(mode)
u_int mode;
{
    switch (mode & S_IFMT) {
    case S_IFDIR:
	putchar(f_bracket ? ']' : '/');
	return (1);
    case S_IFIFO:
	putchar('|');
	return (1);
    case S_IFLNK:
	putchar('@');
	return (1);
    case S_IFSOCK:
	putchar('=');
	return (1);
    }
    if (mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
	putchar('*');
	return (1);
    }
    return (0);
}

static void printlink(p)
FTSENT *p;
{
    int lnklen;
    char name[MAXPATHLEN + 1], path[MAXPATHLEN + 1];

    if (p->fts_level == FTS_ROOTLEVEL)
	snprintf(name, sizeof(name), "%s", p->fts_name);
    else
	snprintf(name, sizeof(name),
		 "%s/%s", p->fts_parent->fts_accpath, p->fts_name);
    if ((lnklen = readlink(name, path, sizeof(path) - 1)) == -1) {
	fprintf(stderr, "\nls: %s: %s\n", name, strerror(errno));
	return;
    }
    path[lnklen] = '\0';
    printf(" -> %s", path);
}
