/*
 * Copyright (c) 1989, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Kevin Fall.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] =
"@(#) Copyright (c) 1989, 1993\n\
	The Regents of the University of California.  All rights reserved.\n";
#endif /* not lint */

#ifndef lint
#if 0
static char sccsid[] = "@(#)mknod.c	8.1 (Berkeley) 6/5/93";
#else
static const char rcsid[] =
    "$Id: mknod.c,v 1.3.2.2 1997/06/18 06:13:34 charnier Exp $";
#endif
#endif /* not lint */

#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <err.h>

#ifdef LINUX
#include <linux/kdev_t.h>
#endif

char *__progname;


int
main(argc, argv)
	int argc;
	char **argv;
{
	dev_t dev;
	char *endp;
	long major, minor;
	mode_t mode;
	int range_error;
	int need;

	__progname = basename(argv[0]);

	mode = 0666;
	if (argv[2][0] == 'c' || argv[2][0] == 'u') {
		mode |= S_IFCHR;
		need = 5;
	}
	else if (argv[2][0] == 'b') {
		mode |= S_IFBLK;
		need = 5;
	}
	else if (argv[2][0] == 'p') {
		mode |= S_IFIFO;
		need = 3;
	}
	else
		errx(1, "node must be type 'b', 'c', or 'p'");
	if (argc != need) {
		fprintf(stderr, "usage: %s path %s\n",
				__progname,
				(need == 5) ? "[bc] major minor"
					    :  "p");
		exit(1);
	}

	if (mode & S_IFIFO) {
		if (mkfifo(argv[1], S_IRWXU | S_IRWXG | S_IRWXO) != 0)
			err(1, "%s", argv[1]);
	}
	else {
		errno = 0;
		major = (long)strtoul(argv[3], &endp, 0);
		if (endp == argv[3] || *endp != '\0')
			errx(1, "%s: non-numeric major number", argv[3]);
		range_error = errno;
		errno = 0;
		minor = (long)strtoul(argv[4], &endp, 0);
		if (endp == argv[4] || *endp != '\0')
			errx(1, "%s: non-numeric minor number", argv[4]);
		range_error |= errno;
#if 0
		dev = makedev(major, minor);
		if (range_error || major(dev) != major || minor(dev) != minor)
#else
		dev = MKDEV(major, minor);
		if (range_error || MAJOR(dev) != major || MINOR(dev) != minor)
#endif
			errx(1, "major or minor number too large");

		if (mknod(argv[1], mode, dev) != 0)
			err(1, "%s", argv[1]);
	}
	exit(0);
}
