/* 
 * Ntp: the interface to /etc/ntp.conf
 *
 * Copyright (c) 2000 David L Parsons 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by David L Parsons (orc@pell.chi.il.us).  My name may not be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.  THIS SOFTWARE IS PROVIDED
 * ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 */
static const char rcsid[] = "$Id: Ntp.cc,v 1.8 1998/06/23 21:53:36 orc Exp $";

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <unistd.h>
#include <ctype.h>
}
#include <string.h>

#include "Ntp.H"
#include "util.H"

/*
 * load up host.conf, or suitable defaults
 */
Ntp::Ntp():KAarray()
{
    Refresh();
    changed(0);
} /* Ntp::Ntp */

/*
 * if anything has changed, write it to disk.
 * 
 * TODO: file locking, dirty file checking.
 */
int
Ntp::commit()
{
    FILE *cf;

    if (changed()) {
	if (backup(NTP_CONF) != 0)
	    return EOF;
	if ((cf = fopen(NTP_CONF, "w")) == (FILE*)0)
	    return EOF;
	make_header(cf, 0);
	print(cf);
	fclose(cf);
	KAarray::changed(0);
    }
    return 0;
} /* Ntp::commit */


/*
 * has the disk configuration file been touched since we last loaded it in?
 */
int
Ntp::dirty()
{
    return 0;
}

/*
 * Refresh() the configuration from the disk image 
 */
int
Ntp::Refresh()
{
    FILE *cf;

    if ((cf = fopen(NTP_CONF, "r")) != (FILE*)0) {
	KAarray::Erase();		/* never merge configurations */
	KAarray::autoappend(1);
	read(cf);
	KAarray::changed(0);
	fclose(cf);
	return 0;
    }
    return EOF;
} /* Ntp::Refresh */


/*
 * read() a configuration file from the given file descriptor.
 */
int
Ntp::read(FILE *in)
{
    char bfr[200];
    char *token;
    char *ptr;
    char *f, *t;

    while (fgets(bfr, sizeof bfr, in) != 0) {
	if (ptr = strchr(bfr, '#'))
	    *ptr = 0;
	if (ptr = strchr(bfr, '\n'))
	    *ptr = 0;

	token = strtok(bfr, "\t ");

	if (token && *token) {
	    ptr = strtok(NULL, "\n");

	    drop(token);
	    if (ptr) {
		/* strip spaces out of the token */
		for (t = f = ptr; *f; f++)
		    if (!isspace(*f))
			*t++ = *f;
		*t = 0;

		create(token, ptr);
	    }
	}
    }
    return count();
} /* Ntp::read */


/*
 * print() writes the contents of host.conf out to the given FILE*
 */
int
Ntp::print(FILE *out)
{
    unsigned int i;

    for (i=0; i<count(); i++) {
	if (strcmp(_list[i]._key, "server") == 0) {
	    for (int j=0; j < _list[i]._content->count(); j++)
		fprintf(out, "%s %s\n", _list[i]._key,
					(*_list[i]._content)[j]);
	}
	else {
	    fprintf(out, "%s", _list[i]._key);
	    for (int j=0; j < _list[i]._content->count(); j++)
		fprintf(out, " %s", (*_list[i]._content)[j]);
	    fputc('\n', out);
	}
    }
    return ferror(out);
} /* Ntp::print */
