/*
 *   Copyright (c) 1996 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

static const char rcsid[] = "$Id: s.validators.c 1.2 98/04/22 03:41:29-00:00 orc $";

/*
 * validators: various validations functions
 *
 * isaddr(): check that an address is a dotted quad
 * isaddrc(): check that an address is a dotten quad or a dotted quad with
 *            a fieldwidth.
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "validators.h"


char *
isaddr(char *candidate, int *dummy)
{
    unsigned int x1, x2, x3, x4;

    if (sscanf(candidate, "%d.%d.%d.%d", &x1, &x2, &x3, &x4) != 4)
	return "field must be a ip address";
    if (x1 > 255 || x2 > 255 || x3 > 255 || x4 > 255)
	return "badly formatted ip address";

    return (char*)0;
}

char *
isaddrc(char *candidate, int *fieldwidth)
{
    char *temp = alloca(strlen(candidate)+1);
    char *p;

    *fieldwidth = 0;
    sprintf(temp, candidate);

    if ((p = strchr(temp, '/')) != (char*)0) {
	*p++ = 0;
	*fieldwidth = atoi(p);

	if (*fieldwidth == 0 || *fieldwidth > 31)
	    return "CIDR width must be >0 and <31";
    }
    return isaddr(temp, 0);
}


char *
isquad(char *candidate, int *dummy)
{
    unsigned int x1, x2, x3, x4;

    if (sscanf(candidate, "%d.%d.%d.%d", &x1, &x2, &x3, &x4) != 4)
	return "field must be a dotted quadword";
    return (char*)0;
}
