/*
 * Functions to implement the various sticks one might find
 * while wandering around the dungeon.
 */

#include <curses.h>
#include <ctype.h>
#include "rogue.h"

fix_stick(cur)
register struct object *cur;
{
    if (strcmp(ws_type[cur->o_which], "staff") == 0) {
	cur->o_weight = 100;
	cur->o_charges = 5 + rnd(10);
	cur->o_damage = "2d3";
	switch (cur->o_which) {
	    when WS_HIT:
		cur->o_hplus = 3;
		cur->o_dplus = 3;
		cur->o_damage = "2d8";
	    when WS_LIGHT:
		cur->o_charges = 20 + rnd(10);
	    }
    }
    else {
	cur->o_damage = "1d3";
	cur->o_weight = 60;
	cur->o_charges = 3 + rnd(5);
	switch (cur->o_which) {
	    when WS_HIT:
		cur->o_hplus = 3;
		cur->o_dplus = 3;
		cur->o_damage = "1d8";
	    when WS_LIGHT:
		cur->o_charges = 10 + rnd(10);
	    }
    }
    cur->o_hurldmg = "1d1";

}

do_zap(gotdir, which, blessed)
bool gotdir;
int which;
bool blessed;
{
    register struct linked_list *item, *nitem;
    register struct object *obj, *nobj;
    register struct room *rp;
    register struct thing *tp;
    register int y, x;
    bool cursed, is_stick;

    cursed = FALSE;
    is_stick = FALSE;

    if (which == NULL) {
	if ((item = get_item("zap with", STICK)) == NULL)
	    return;
	obj = OBJPTR(item);
	if (obj->o_type != STICK) {
	    msg("You can't zap with that!");
	    return;
	}
	which = obj->o_which;
	ws_know[which] = TRUE;
	if (obj->o_charges < 1)
	{
	    msg("Nothing happens.");
	    return;
	}
	obj->o_charges--;
	if (!gotdir)
	    do {
		delta.y = rnd(3) - 1;
		delta.x = rnd(3) - 1;
	    } while (delta.y == 0 && delta.x == 0);

	cursed = obj->o_flags & ISCURSED;
	blessed = obj->o_flags & ISBLESSED;
	is_stick = TRUE;
    }
    switch (which)
    {
	when WS_LIGHT:
	    /*
	     * Reddy Kilowat wand.  Light up the room
	     */
	    blue_light(blessed, cursed);
	when WS_DRAIN:
	    /*
	     * Take away 1/2 of hero's hit points, then take it away
	     * from each of the monsters in the room or next to hero
	     * if he is in a passage (but leave the monsters alone
	     * if the stick is cursed)
	     */
	    if (pstats.s_hpt < 2) {
		msg("You are too weak to use it.");
		return;
	    }
	    if (obj->o_flags & ISCURSED)
		pstats.s_hpt /= 2;
	    else if ((rp = roomin(&hero)) == NULL)
		drain(hero.y-1, hero.y+1, hero.x-1, hero.x+1);
	    else
		drain(rp->r_pos.y, rp->r_pos.y+rp->r_max.y,
		    rp->r_pos.x, rp->r_pos.x+rp->r_max.x);
	when WS_POLYMORPH:
	case WS_TELMON:
	case WS_CANCEL:
	{
	    register char monster, oldch;
	    register int rm;

	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x))) {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(monster = mvwinch(mw, y, x))) {
		item = find_mons(y, x);
		tp = THINGPTR(item);
		/* if the monster gets the saving throw, leave the case */
		if (save_throw(VS_MAGIC, tp)) {
		    msg("Nothing happens.");
		    break;
		}

		/* Unhold player */
		if (on(*tp, DIDHOLD)) {
		    turn_off(*tp, DIDHOLD);
		    if (--hold_count == 0) turn_off(player, ISHELD);
		}
		/* unsuffocate player */
		if (on(*tp, DIDSUFFOCATE)) {
		    turn_off(*tp, DIDSUFFOCATE);
		    extinguish(suffocate);
		}

		if (which == WS_POLYMORPH) {
		    detach(mlist, item);
		    oldch = tp->t_oldch;
		    delta.y = y;
		    delta.x = x;
		    new_monster(item, rnd(NUMMONST-1) + 1, &delta, FALSE);
		    monster = tp->t_type;
		    if (off(*tp, ISRUN))
			runto(&delta, &hero);
		    if (isalpha(mvwinch(cw, y, x)))
			mvwaddch(cw, y, x, monster);
		    tp->t_oldch = oldch;
		    msg(terse ? "A new %s!" : "You have created a new %s!",
			monsters[tp->t_index].m_name);
		}
		else if (which == WS_CANCEL) {
		    tp->t_flags[0] &= CANC0MASK;
		    tp->t_flags[1] &= CANC1MASK;
		    tp->t_flags[2] &= CANC2MASK;
		    tp->t_flags[3] &= CANC3MASK;
		}
		else { /* A teleport stick */
		    if (cursed) {	/* Teleport monster to player */
			if ((y == (hero.y + delta.y)) &&
			    (x == (hero.x + delta.x)))
				msg("Nothing happens.");
			else {
			    tp->t_pos.y = hero.y + delta.y;
			    tp->t_pos.x = hero.x + delta.x;
			}
		    }
		    else if (blessed) {	/* Get rid of monster */
			killed(item, FALSE, FALSE);
			return;
		    }
		    else {
			register int i=0;

			do {	/* Move monster to another room */
			    rm = rnd_room();
			    rnd_pos(&rooms[rm], &tp->t_pos);
			} until(winat(tp->t_pos.y,tp->t_pos.x) ==
			        FLOOR || i++>500);
		    }

		    /* Now move the monster */
		    if (isalpha(mvwinch(cw, y, x)))
			mvwaddch(cw, y, x, tp->t_oldch);
		    tp->t_dest = &hero;
		    turn_on(*tp, ISRUN);
		    turn_off(*tp, ISDISGUISE);
		    mvwaddch(mw, y, x, ' ');
		    mvwaddch(mw, tp->t_pos.y, tp->t_pos.x, monster);
		    if (tp->t_pos.y != y || tp->t_pos.x != x)
			tp->t_oldch = mvwinch(cw, tp->t_pos.y, tp->t_pos.x);
		}
	    }
	}
	when WS_MISSILE:
	{

	    static struct object bolt =
	    {
		'*' , {0, 0}, "", 0, 0, "1d4" , 0, 0, 100, 1
	    };

	    sprintf(bolt.o_hurldmg, "%dd4", pstats.s_lvl);
	    do_motion(&bolt, delta.y, delta.x, &player);
	    if (!hit_monster(unc(bolt.o_pos), &bolt, &player))
	       msg("The missle vanishes with a puff of smoke");
	}
	when WS_HIT:
	{
	    register char ch;
	    struct object strike; /* don't want to change sticks attributes */

	    delta.y += hero.y;
	    delta.x += hero.x;
	    ch = winat(delta.y, delta.x);
	    if (isalpha(ch))
	    {
		strike = *obj;
		strike.o_hplus  = 6;
		if (strcmp(ws_type[which], "staff") == 0)
		    strike.o_damage = "3d8";
		else
		    strike.o_damage = "2d8";
		fight(&delta, ch, &strike, FALSE);
	    }
	}
	case WS_SLOW_M:
	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x)))
	    {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(mvwinch(mw, y, x)))
	    {

		item = find_mons(y, x);
		tp = THINGPTR(item);
		if (cursed)
		{
		    if (on(*tp, ISSLOW))
			turn_off(*tp, ISSLOW);
		    else
			turn_on(*tp, ISHASTE);
		}
		else if (blessed) {
		    turn_off(*tp, ISRUN);

		    turn_on(*tp, ISHELD);
		    return;
		}
		else {
		    if (on(*tp, ISHASTE))
			turn_off(*tp, ISHASTE);
		    else
			turn_on(*tp, ISSLOW);
		    tp->t_turn = TRUE;
		}
		delta.y = y;
		delta.x = x;
		runto(&delta, &hero);
	    }
	when WS_CHARGE:
	    if (ws_know[WS_CHARGE] != TRUE && is_stick)
		msg("This is a wand of charging.");
	    if ((nitem = get_item("charge", STICK)) != NULL) {
		nobj = OBJPTR(nitem);
		if ((++nobj->o_charges == 1) && (nobj->o_which == WS_HIT))
		    fix_stick(nobj);
	    }
	when WS_ELECT:
	case WS_FIRE:
	case WS_COLD:
	    {
		char *name;

		if (which == WS_ELECT)
		    name = "lightning bolt";
		else if (which == WS_FIRE)
		    name = "flame";
		else
		    name = "ice";

		shoot_bolt(&player, hero, delta, TRUE, D_BOLT, name, roll(6,6));
	    }
	when WS_ANTIM: {
	    register int m1, m2, x1, y1;
	    register char ch;
	    register struct linked_list *ll;
	    register struct thing *lt;

	    y1 = hero.y;
	    x1 = hero.x;
	    do {
		y1 += delta.y;
		x1 += delta.x;
		ch = winat(y1,x1);
	    } while (ch == PASSAGE || ch == FLOOR);
	    for (m1 = x1 - 1 ; m1 <= x1 + 1 ; m1++) {
		for(m2 = y1 - 1 ; m2 <= y1 + 1 ; m2++) {
		    ch = winat(m2,m1);
		    if (m1 == hero.x && m2 == hero.y)
			continue;
		    if (ch != ' ') {
			ll = find_obj(m2,m1);
			if (ll != NULL) {
			    detach(lvl_obj,ll);
			    discard(ll);
			}
			ll = find_mons(m2,m1);
			if (ll != NULL) {
			    lt = THINGPTR(ll);
			    check_residue(lt);
			    detach(mlist,ll);
			    discard(ll);
			    mvwaddch(mw,m2,m1,' ');
			}
			mvaddch(m2,m1,' ');
			mvwaddch(cw,m2,m1,' ');
		    }
		}
	    }
	    touchwin(cw);
	    touchwin(mw);
	}
	when WS_CONFMON:
	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x)))
	    {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(mvwinch(mw, y, x)))
	    {
		item = find_mons(y, x);
		tp = THINGPTR(item);
		if (save_throw(VS_MAGIC, tp))
		     msg("nothing happens");
		else
		     turn_on (*tp, ISHUH);
		delta.y = y;
		delta.x = x;
		runto(&delta, &hero);
	    }
	when WS_PARALYZE:
	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x)))
	    {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(mvwinch(mw, y, x)))
	    {
		item = find_mons(y, x);
		tp = THINGPTR(item);
		if (save_throw(VS_WAND, tp))
		    msg("nothing happens");
		else {
		    tp->t_no_move = FREEZETIME;
		}
	    }
	when WS_MDEG:
	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x)))
	    {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(mvwinch(mw, y, x)))
	    {
		item = find_mons(y, x);
		tp = THINGPTR(item);
		if (cursed) {
		     tp->t_stats.s_hpt *= 2;
		     msg("The %s appears to be stronger now!", 
			monsters[tp->t_index].m_name);
		}
		else {
		     tp->t_stats.s_hpt /= 2;
		     msg("The %s appears to be weaker now", 
			monsters[tp->t_index].m_name);
		}
		delta.y = y;
		delta.x = x;
		runto(&delta, &hero);
	        if (tp->t_stats.s_hpt < 1)
		     killed(item, TRUE, TRUE);
	    }
	when WS_ANNIH:
	    y = hero.y;
	    x = hero.x;
	    while (shoot_ok(winat(y, x))) {
		y += delta.y;
		x += delta.x;
	    }
	    if (isalpha(mvwinch(mw, y, x))) {
		item = find_mons(y, x);
		tp = THINGPTR(item);
		if (cursed) {
		    register int m1, m2;
		    coord mp;
		    struct linked_list *titem;
		    char ch;
		    struct thing *th;
		    for (m1=tp->t_pos.x-1 ; m1 <= tp->t_pos.x+1 ; m1++) {
			for(m2=tp->t_pos.y-1 ; m2<=tp->t_pos.y+1 ; m2++) {
			    ch = winat(m2,m1);
			    if (shoot_ok(ch) && ch != PLAYER) {
				mp.x = m1;	/* create it */
				mp.y = m2;
				titem = new_item(sizeof(struct thing));
				new_monster(titem,(short)tp->t_index,&mp,FALSE);
				th = THINGPTR(titem);
				turn_on (*th, ISMEAN);
				runto(&mp,&hero);
			    }
			}
		    }
		    delta.y = y;
		    delta.x = x;
		    turn_on (*tp, ISMEAN);
		    runto(&delta, &hero);
		}
		else { /* if its a UNIQUE it might still live */
		    tp = THINGPTR(item);
		    if (on(*tp, ISUNIQUE) && save_throw(VS_MAGIC, tp)) {
			tp->t_stats.s_hpt /= 2;
			if (tp->t_stats.s_hpt < 1) {
			     killed(item, FALSE, TRUE);
			     msg("You have disintegrated the %s", 
				    monsters[tp->t_index].m_name);
			}
			else {
			    delta.y = y;
			    delta.x = x;
			    runto(&delta, &hero);
			    msg("The %s appears wounded",
				monsters[tp->t_index].m_name);
			}
		    }
		    else {
			msg("You have disintegrated the %s", 
				monsters[tp->t_index].m_name);
			killed (item, FALSE, TRUE);
		    }
		}
	    }
	otherwise:
	    msg("What a bizarre schtick!");
    }
}

/*
 * drain:
 *	Do drain hit points from player shtick
 *  changed by tsc to zot them all with what the player was zotted.
 *	otherwise draining party rooms is pretty useless.
 */


drain(ymin, ymax, xmin, xmax)
int ymin, ymax, xmin, xmax;
{
    register int i, j, count;
    register struct thing *ick;
    register struct linked_list *item;


    /*
     * First count how many things to see whether we zot them
     */

    count = 0;
    for (i = ymin; i <= ymax; i++)
	for (j = xmin; j <= xmax; j++)
	    if (isalpha(mvwinch(mw, i, j)))
		count++;
    if (count == 0)
    {
	msg("You have a tingling feeling");
	return;
    }

	count = (pstats.s_hpt /= 2);

    /*
     * Now zot all of the monsters
     */
    for (i = ymin; i <= ymax; i++)
	for (j = xmin; j <= xmax; j++)
	    if (isalpha(mvwinch(mw, i, j)) &&
	        ((item = find_mons(i, j)) != NULL))
	    {
		ick = THINGPTR(item);
		if ((ick->t_stats.s_hpt -= count) < 1)
		    killed(item, cansee(i, j) && !on(*ick, ISINVIS), TRUE);
	    }
}

/*
 * charge a wand for wizards.
 */
char *
charge_str(obj)
register struct object *obj;
{
    static char buf[20];

    if (!(obj->o_flags & ISKNOW))
	buf[0] = '\0';
    else if (terse)
	sprintf(buf, " [%d]", obj->o_charges);
    else
	sprintf(buf, " [%d charges]", obj->o_charges);
    return buf;
}


/*
 * shoot_bolt fires a bolt from the given starting point in the
 * 	      given direction
 */

bool
shoot_bolt(shooter, start, dir, get_points, reason, name, damage)
struct thing *shooter;
coord start, dir;
bool get_points;
short reason;
char *name;
int damage;
{
    register char dirch, ch;
    register bool used, change;
    register short y, x;
    coord pos;
    coord spotpos[BOLT_LENGTH];
    bool ret_val = FALSE;	/* True if monster gets killed */

    switch (dir.y + dir.x) {
	when 0: dirch = '/';
	when 1: case -1: dirch = (dir.y == 0 ? '-' : '|');
	when 2: case -2: dirch = '\\';
    }
    pos.y = start.y + dir.y;
    pos.x = start.x + dir.x;

    used = FALSE;
    change = FALSE;
    for (y = 0; y < BOLT_LENGTH && !used; y++)
    {
	ch = winat(pos.y, pos.x);
	spotpos[y] = pos;

	/* Are we at hero? */
	if (ce(pos, hero)) goto at_hero;

	switch (ch)
	{
	    case SECRETDOOR:
	    case '|':
	    case '-':
	    case ' ':
		if (dirch == '-' || dirch == '|') {
		    dir.y = -dir.y;
		    dir.x = -dir.x;
		}
		else switch (ch) {
		    case '|':
		    case '-':
		    case SECRETDOOR:

			{
			    register struct room *rp;

			    rp = roomin(&pos);
			    if (pos.y == rp->r_pos.y ||
				pos.y == rp->r_pos.y + rp->r_max.y - 1) {

				dir.y = -dir.y;
				change ^= TRUE;
			    }
			    if (pos.x == rp->r_pos.x ||
				pos.x == rp->r_pos.x + rp->r_max.x - 1) {
				dir.x = -dir.x;
				change ^= TRUE;
			    }
			}
			break;
		    default:	/* A wall */
			{
			    char chy = mvinch(pos.y-dir.y, pos.x+dir.x),
				 chx = mvinch(pos.y+dir.y, pos.x-dir.x);

			    if (chy != WALL && chy != SECRETDOOR &&
				chy != '-' && chy != '|') {
				dir.y = -dir.y;
				change = TRUE;
			    }
			    else if (chx != WALL && chx != SECRETDOOR &&
				     chx != '-' && chx != '|') {
				    dir.x = -dir.x;
				    change = TRUE;
				}
			    else {
				dir.y = -dir.y;
				dir.x = -dir.x;
			    }
			}
		}

		/* Do we change how the bolt looks? */
		if (change) {
		    change = FALSE;
		    if (dirch == '\\') dirch = '/';
		    else if (dirch == '/') dirch = '\\';
		}

		y--;
		msg("The %s bounces", name);
		break;
	    default:
		if (isalpha(ch)) {
		    register struct linked_list *item;
		    register struct thing *tp;
		    register char *mname;

		    item = find_mons(unc(pos));
		    tp = THINGPTR(item);
		    mname = monsters[tp->t_index].m_name;

		    if (!save_throw(VS_MAGIC, tp)) {
			if (on(*tp, ISDISGUISE) &&
			    (tp->t_type != tp->t_disguise) &&
			    off(player, ISBLIND)) {
			    msg("Wait! That's a %s!", mname);
			    turn_off(*tp, ISDISGUISE);
			}

			tp->t_wasshot = TRUE;

			/* Hit the monster -- does it do damage? */
			if ((strcmp(name, "ice") == 0 && on(*tp, NOCOLD)) ||
			    (strcmp(name, "ice") == 0 && on(*tp, ISUNDEAD)) ||
			    (strcmp(name,"flame")== 0 && on(*tp, NOFIRE)) ||
			    (strcmp(name,"lightning bolt")==0 && 
							on(*tp,NOBOLT))) {
			    msg("The %s has no effect on the %s.", name,
				on(player, ISBLIND) ? "monster" : mname);
			}

			else if ((strcmp(name, "lightning bolt") == 0) &&
				 on(*tp, BOLTDIVIDE)) {
				if (creat_mons(tp, tp->t_index, FALSE))
				    msg("The %s divides the %s.", name, mname);
				else msg("The %s has no effect on the %s.",
					name, mname);
			}

			else if ((tp->t_stats.s_hpt -= damage) <= 0) {
			    msg("The %s kills the %s", name,
				on(player, ISBLIND) ? "monster" : mname);
			    killed(item, FALSE, get_points);
			    ret_val = TRUE;
			}

			else {
			    msg("The %s hits the %s", name,
				on(player, ISBLIND) ? "monster" : mname);
			    if (get_points) runto(&pos, &hero);
			}
			used = TRUE;
		    }
		    else if (isalpha(show(pos.y, pos.x))) {
			if (terse)
			    msg("%s misses", name);
			else
			    msg("The %s whizzes past the %s", name,
			     monsters[id_monst(show(pos.y, pos.x))].m_name);
			if (get_points) runto(&pos, &hero);
		    }
		}
		else if (pos.y == hero.y && pos.x == hero.x) {
at_hero: 	    if (!save(VS_MAGIC)) {
			if (terse)
			    msg("The %s hits you", name);
			else
			    msg("You are hit by the %s", name);
			if ((pstats.s_hpt -= damage) <= 0)
			    death(reason);
			used = TRUE;

			/* Check for gas with special effects */
			if (!save(VS_BREATH)) {
			    if (strcmp(name, "nerve gas") == 0) {
				if (no_command == 0) {
				    msg("The nerve gas paralyzes you.");
				    no_command = FREEZETIME;
				}
			    }
			    else if (strcmp(name, "sleeping gas") == 0) {
				if (no_command == 0) {
				    msg("The sleeping gas puts you to sleep.");
				    no_command = SLEEPTIME;
				}
			    }
			    else if (strcmp(name, "slow gas") == 0) {
				msg("You feel yourself moving %sslower.",
					on(player, ISSLOW) ? "even " : "");
				if (on(player, ISSLOW))
				    lengthen(noslow, rnd(10) + 4);
				else {
				    turn_on(player, ISSLOW);
				    player.t_turn = TRUE;
				    fuse(noslow, 0, rnd(10)+4, AFTER);
				}
			    }
			    else if (strcmp(name, "fear gas") == 0) {
				if (!(on(player, ISFLEE) &&
				      (player.t_dest == &shooter->t_pos)) &&
				    (shooter != &player)) {
				    turn_on(player, ISFLEE);
				    player.t_dest = &shooter->t_pos;
				    msg("The fear gas terrifies you.",
					monsters[shooter->t_index].m_name);
				}

			    }
			}
		    }
		    else
			msg("The %s whizzes by you", name);
		}
		mvwaddch(cw, pos.y, pos.x, dirch);
		draw(cw);
	}
	pos.y += dir.y;
	pos.x += dir.x;
    }
    for (x = 0; x < y; x++)
	mvwaddch(cw, spotpos[x].y, spotpos[x].x,
			show(spotpos[x].y, spotpos[x].x));

    return(ret_val);
}
