/*
 * Contains functions for dealing with things like
 * potions and scrolls
 *

 */

#include <curses.h>
#include <ctype.h>
#include "rogue.h"

/*
 * inv_name:
 *	return the name of something as it would appear in an
 *	inventory.
 */
char *
inv_name(obj, drop)
register struct object *obj;
register bool drop;
{
    register char *pb;

    switch(obj->o_type)
    {
	when SCROLL:
	    if (obj->o_count == 1)
		 sprintf(prbuf, "A %sscroll ", blesscurse(obj->o_flags));
	    else
		 sprintf(prbuf, "%d %sscrolls ", 
			obj->o_count, blesscurse(obj->o_flags));
	    pb = &prbuf[strlen(prbuf)];
	    if (s_know[obj->o_which] || (obj->o_flags & ISPOST))
		sprintf(pb, "of %s", s_magic[obj->o_which].mi_name);
	    else if (s_guess[obj->o_which])
		sprintf(pb, "called %s", s_guess[obj->o_which]);
	    else
		sprintf(pb, "titled '%s'", s_names[obj->o_which]);
        when POTION:
	    if (obj->o_count == 1)
		 sprintf(prbuf, "A %spotion ", blesscurse(obj->o_flags));
	    else
		 sprintf(prbuf, "%d %spotions ", 
			obj->o_count, blesscurse(obj->o_flags));
	    pb = &prbuf[strlen(prbuf)];
	    if (p_know[obj->o_which] || (obj->o_flags & ISPOST))
		sprintf(pb, "of %s(%s)", p_magic[obj->o_which].mi_name,
		    p_colors[obj->o_which]);
	    else if (p_guess[obj->o_which])
		sprintf(pb, "called %s(%s)", p_guess[obj->o_which],
		    p_colors[obj->o_which]);
	    else {
		if (obj->o_count == 1)
		    sprintf(prbuf, "A%s %s potion",
			    vowelstr(p_colors[obj->o_which]),
			    p_colors[obj->o_which]);
		else
		    sprintf(prbuf, "%d %s potions",
			    obj->o_count, p_colors[obj->o_which]);
	    }
	when FOOD:
	    if (obj->o_which == 1)
		if (obj->o_count == 1)
		    sprintf(prbuf, "A%s %s", vowelstr(fruit), fruit);
		else
		    sprintf(prbuf, "%d %ss", obj->o_count, fruit);
	    else
		if (obj->o_count == 1)
		    strcpy(prbuf, "Some food");
		else
		    sprintf(prbuf, "%d rations of food", obj->o_count);
	when WEAPON:
	    if (obj->o_count > 1)
		sprintf(prbuf, "%d ", obj->o_count);
	    else
		strcpy(prbuf, "A ");
	    pb = &prbuf[strlen(prbuf)];
	    if ((obj->o_flags & ISKNOW) || (obj->o_flags & ISPOST))
		sprintf(pb, "%s %s", num(obj->o_hplus, obj->o_dplus),
		    weaps[obj->o_which].w_name);
	    else
		sprintf(pb, "%s", weaps[obj->o_which].w_name);
	    if (obj->o_count > 1)
		strcat(prbuf, "s");
	when ARMOR:
	    if ((obj->o_flags & ISKNOW) || (obj->o_flags & ISPOST))
		sprintf(prbuf, "%s %s",

		    num(armors[obj->o_which].a_class - obj->o_ac, 0),
		    armors[obj->o_which].a_name);
	    else
		sprintf(prbuf, "%s", armors[obj->o_which].a_name);
	when AMULET:
	    strcpy(prbuf, "The Amulet of Yendor");
	when STICK:
	    sprintf(prbuf, "A %s%s ", 
		blesscurse(obj->o_flags), ws_type[obj->o_which]);
	    pb = &prbuf[strlen(prbuf)];
	    if (ws_know[obj->o_which] || (obj->o_flags & ISPOST))

		sprintf(pb, "of %s%s(%s)", ws_magic[obj->o_which].mi_name,
		    charge_str(obj), ws_made[obj->o_which]);
	    else if (ws_guess[obj->o_which])
		sprintf(pb, "called %s(%s)", ws_guess[obj->o_which],
		    ws_made[obj->o_which]);
	    else

		sprintf(&prbuf[2], "%s %s", ws_made[obj->o_which],
		    ws_type[obj->o_which]);
        when RING:
	    if (r_know[obj->o_which] || (obj->o_flags & ISPOST))
		sprintf(prbuf, "A%s ring of %s(%s)", ring_num(obj),
		    r_magic[obj->o_which].mi_name, r_stones[obj->o_which]);
	    else if (r_guess[obj->o_which])
		sprintf(prbuf, "A ring called %s(%s)",
		    r_guess[obj->o_which], r_stones[obj->o_which]);

	    else
		sprintf(prbuf, "A%s %s ring", vowelstr(r_stones[obj->o_which]),
		    r_stones[obj->o_which]);
	otherwise:
	    debug("Picked up something funny");
	    sprintf(prbuf, "Something bizarre %s", unctrl(obj->o_type));
    }

    /* Is it marked? */
    if (obj->o_mark[0]) {
	pb = &prbuf[strlen(prbuf)];
	sprintf(pb, " <%s>", obj->o_mark);
    }

    if (obj == cur_armor)
	strcat(prbuf, " (being worn)");
    if (obj == cur_weapon)
	strcat(prbuf, " (weapon in hand)");
    if      (obj == cur_ring[LEFT_1])  strcat(prbuf, " (on left hand)");
    else if (obj == cur_ring[LEFT_2])  strcat(prbuf, " (on left hand)");
    else if (obj == cur_ring[LEFT_3])  strcat(prbuf, " (on left hand)");
    else if (obj == cur_ring[LEFT_4])  strcat(prbuf, " (on left hand)");
    else if (obj == cur_ring[RIGHT_1]) strcat(prbuf, " (on right hand)");
    else if (obj == cur_ring[RIGHT_2]) strcat(prbuf, " (on right hand)");
    else if (obj == cur_ring[RIGHT_3]) strcat(prbuf, " (on right hand)");

    else if (obj == cur_ring[RIGHT_4]) strcat(prbuf, " (on right hand)");
    if (obj->o_flags & ISPROT)
	strcat(prbuf, " [protected]");
    if (drop && isupper(prbuf[0]))
	prbuf[0] = tolower(prbuf[0]);
    else if (!drop && islower(*prbuf))
	*prbuf = toupper(*prbuf);
    if (!drop)
	strcat(prbuf, ".");
    return prbuf;
}

/*
 * drop:
 *	put something down
 */
drop(item)
struct linked_list *item;
{
    register char ch;
    register struct linked_list *obj, *nobj;
    register struct object *op;

    if (item == NULL) {
	ch = mvwinch(stdscr, hero.y, hero.x);
	if (ch != FLOOR && ch != PASSAGE && ch != POOL) {
	    msg("There is something there already");
	    return(FALSE);
	}
	if ((obj = get_item("drop", 0)) == NULL)
	    return(FALSE);
    }
    else {
	obj = item;
    }
    op = (struct object *) ldata(obj);
    if (!dropcheck(op))
	return(FALSE);

    /*
     * If it is a scare monster scroll, curse it
     */
    if (op->o_type == SCROLL && op->o_which == S_SCARE) {
	if (op->o_flags & ISBLESSED)
	    op->o_flags &= ~ISBLESSED;
	else op->o_flags |= ISCURSED;
    }

    /*
     * Take it out of the pack
     */
    if (op->o_count >= 2 && op->o_group == NULL)
    {
	nobj = new_item(sizeof *op);
	op->o_count--;
	op = (struct object *) ldata(nobj);
	*op = *((struct object *) ldata(obj));
	op->o_count = 1;
	obj = nobj;
    }
    else {
	detach(pack, obj);
        inpack--;
	freeletter(obj);
    }
    if(ch == POOL) {
	msg("Your %s sinks out of sight.",inv_name(op,TRUE));
	discard(obj);
    }
    else if (levtype == POSTLEV) {
	op->o_pos = hero;	/* same place as hero */
	fall(obj,FALSE);
	if (item == NULL)	/* if item wasn't sold */
	    msg("Thanks for your donation to the fiend's flea market.");
    }
    else {
	/*
	 * Link it into the level object list
	 */
	attach(lvl_obj, obj);
	mvaddch(hero.y, hero.x, op->o_type);
	op->o_pos = hero;
	msg("Dropped %s", inv_name(op, TRUE));
    }
    updpack(FALSE);
    return (TRUE);
}

/*
 * do special checks for dropping or unweilding|unwearing|unringing
 */
dropcheck(op)
register struct object *op;
{
    short save_max;


    if (op == NULL)
	return TRUE;
    if (levtype == POSTLEV) {
	if ((op->o_flags & ISCURSED) && (op->o_flags & ISKNOW)) {
	    msg("The trader does not accept your shoddy merchandise");
	    return(FALSE);
	}
    }
    if (op != cur_armor && op != cur_weapon &&
	op != cur_ring[LEFT_1] && op != cur_ring[LEFT_2] &&
	op != cur_ring[LEFT_3] && op != cur_ring[LEFT_4] &&
	op != cur_ring[RIGHT_1] && op != cur_ring[RIGHT_2] &&
	op != cur_ring[RIGHT_3] && op != cur_ring[RIGHT_4]) 
	    return TRUE;
    if (op->o_flags & ISCURSED)
    {
	msg("You can't.  It appears to be cursed.");
	return FALSE;
    }
    if (op == cur_weapon)
	cur_weapon = NULL;
    else if (op == cur_armor)
    {
	waste_time();
	cur_armor = NULL;
    }
    else if (op == cur_ring[LEFT_1] || op == cur_ring[LEFT_2] ||
	op == cur_ring[LEFT_3] || op == cur_ring[LEFT_4] ||
	op == cur_ring[RIGHT_1] || op == cur_ring[RIGHT_2] ||
	op == cur_ring[RIGHT_3] || op == cur_ring[RIGHT_4])
    {
	switch (op->o_which) {
	    when R_ADDSTR:
		save_max = max_stats.s_str;
		chg_str(-op->o_ac);
		max_stats.s_str = save_max;
	    when R_ADDHIT:
		pstats.s_dext -= op->o_ac;
	    when R_ADDINTEL:
		pstats.s_intel -= op->o_ac;
	    when R_ADDWISDOM:
		pstats.s_wisdom -= op->o_ac;
	    when R_SEEINVIS:
		if (find_slot(unsee) == NULL) {
		    turn_off(player, CANSEE);
		    msg("The tingling feeling leaves your eyes");
		}
		light(&hero);
		mvwaddch(cw, hero.y, hero.x, PLAYER);
	    when R_LIGHT: {
		    if(roomin(&hero) != NULL) {
			    light(&hero);
			    mvwaddch(cw, hero.y, hero.x, PLAYER);
		    }
	        }
	}
	if      (op == cur_ring[LEFT_1])  cur_ring[LEFT_1]  = NULL;
	else if (op == cur_ring[LEFT_2])  cur_ring[LEFT_2]  = NULL;
	else if (op == cur_ring[LEFT_3])  cur_ring[LEFT_3]  = NULL;
	else if (op == cur_ring[LEFT_4])  cur_ring[LEFT_4]  = NULL;

	else if (op == cur_ring[RIGHT_1]) cur_ring[RIGHT_1] = NULL;
	else if (op == cur_ring[RIGHT_2]) cur_ring[RIGHT_2] = NULL;
	else if (op == cur_ring[RIGHT_3]) cur_ring[RIGHT_3] = NULL;
	else if (op == cur_ring[RIGHT_4]) cur_ring[RIGHT_4] = NULL;
    }

    return TRUE;
}

/*
 * return a new thing
 */
struct linked_list *
new_thing()
{
    register struct linked_list *item;
    register struct object *cur;
    register int j, k;
    short blesschance, cursechance;

    item = new_item(sizeof *cur);
    cur = (struct object *) ldata(item);
    cur->o_hplus = cur->o_dplus = 0;
    cur->o_damage = cur->o_hurldmg = "0d0";
    cur->o_ac = 11;

    cur->o_count = 1;
    cur->o_group = 0;
    cur->o_flags = 0;
    cur->o_weight = 0;
    cur->o_mark[0] = '\0';
    /*
     * Decide what kind of object it will be
     * If we haven't had food for a while, let it be food.
     */
    blesschance = rnd(100);
    cursechance = rnd(100);
    switch (no_food > 3 ? 2 : pick_one(things, NUMTHINGS))
    {
	when 0:
	    cur->o_type = POTION;
	    cur->o_which = pick_one(p_magic, MAXPOTIONS);
	    cur->o_weight = things[TYP_POTION].mi_wght;
	    if (cursechance < p_magic[cur->o_which].mi_curse)
		cur->o_flags |= ISCURSED;
	    else if (blesschance < p_magic[cur->o_which].mi_bless)
		cur->o_flags |= ISBLESSED;
	when 1:
	    cur->o_type = SCROLL;
	    cur->o_which = pick_one(s_magic, MAXSCROLLS);
	    cur->o_weight = things[TYP_SCROLL].mi_wght;
	    if (cursechance < s_magic[cur->o_which].mi_curse)
		cur->o_flags |= ISCURSED;
	    else if (blesschance < s_magic[cur->o_which].mi_bless)
		cur->o_flags |= ISBLESSED;

	when 2:
	    no_food = 0;
	    cur->o_type = FOOD;
	    cur->o_weight = things[TYP_FOOD].mi_wght;
	    cur->o_count += extras();
	    if (rnd(100) > 10)
		cur->o_which = 0;
	    else
		cur->o_which = 1;
	when 3:
	    cur->o_type = WEAPON;
	    cur->o_which = rnd(MAXWEAPONS);
	    init_weapon(cur, cur->o_which);
	    if (cursechance < 10)
	    {
		short bad=(rnd(10) < 1) ? 2 : 1;


		cur->o_flags |= ISCURSED;
		cur->o_hplus -= bad;
		cur->o_dplus -= bad;
	    }
	    else if (blesschance < 15) {
		short good=(rnd(10) < 1) ? 2 : 1;

		cur->o_hplus += good;
		cur->o_dplus += good;
	    }
	when 4:

	    cur->o_type = ARMOR;
	    for (j = 0; j < MAXARMORS; j++)
		if (blesschance < armors[j].a_prob)
		    break;
	    if (j == MAXARMORS)
	    {
		debug("Picked a bad armor %d", blesschance);
		j = 0;
	    }
	    cur->o_which = j;
	    cur->o_ac = armors[j].a_class;
	    cur->o_weight = armors[j].a_wght;

	    if ((k = rnd(100)) < 20)
	    {
		cur->o_flags |= ISCURSED;
		cur->o_ac += rnd(3)+1;
	    }
	    else if (k < 28)
		cur->o_ac -= rnd(3)+1;
	when 5:
	    cur->o_type = RING;
	    cur->o_which = pick_one(r_magic, MAXRINGS);

	    cur->o_weight = things[TYP_RING].mi_wght;
	    if (cursechance < r_magic[cur->o_which].mi_curse)
		cur->o_flags |= ISCURSED;
	    else if (blesschance < r_magic[cur->o_which].mi_bless)
		cur->o_flags |= ISBLESSED;
	    switch (cur->o_which)
	    {
		when R_ADDSTR:
		case R_ADDWISDOM:
		case R_ADDINTEL:
		case R_PROTECT:
		case R_ADDHIT:
		case R_ADDDAM:
		    cur->o_ac = rnd(2) + 1;	/* From 1 to 3 */
		    if (cur->o_flags & ISCURSED)
			cur->o_ac = -cur->o_ac;
		    if (cur->o_flags & ISBLESSED) cur->o_ac++;
		when R_DIGEST:
		    if (cur->o_flags & ISCURSED) cur->o_ac = -1;
		    else if (cur->o_flags & ISBLESSED) cur->o_ac = 2;
		    else cur->o_ac = 1;
	    }
	when 6:
	    cur->o_type = STICK;
	    cur->o_which = pick_one(ws_magic, MAXSTICKS);
	    fix_stick(cur);
	    if (cursechance < ws_magic[cur->o_which].mi_curse)
		cur->o_flags |= ISCURSED;
	    else if (blesschance < ws_magic[cur->o_which].mi_bless)
		cur->o_flags |= ISBLESSED;
	otherwise:
	    debug("Picked a bad kind of object");
	    wait_for(' ');
    }
    return item;
}

/*
 * provide a new item tailored to specification
 */
struct linked_list *
spec_item(type, which, hit, damage)

int type, which, hit, damage;
{
    register struct linked_list *item;
    register struct object *obj;

    item = new_item(sizeof *obj);
    obj = OBJPTR(item);
    obj->o_count = 1;
    obj->o_group = 0;
    obj->o_type = type;
    obj->o_which = which;
    obj->o_damage = obj->o_hurldmg = "0d0";
    obj->o_hplus = 0;
    obj->o_dplus = 0;
    obj->o_flags = 0;
    obj->o_mark[0] = '\0';
    obj->o_text = NULL;
    obj->o_launch = 0;
    obj->o_weight = 0;

    /* Handle special characteristics */
    switch (type) {
	when WEAPON:
	    init_weapon(obj, which);
	    obj->o_hplus = hit;
	    obj->o_dplus = damage;
	    obj->o_ac = 10;

	    if (hit > 0 || damage > 0) obj->o_flags |= ISBLESSED;
	    else if (hit < 0 || damage < 0) obj->o_flags |= ISCURSED;

	when ARMOR:
	    obj->o_ac = armors[which].a_class - hit;
	    if (hit > 0) obj->o_flags |= ISBLESSED;
	    else if (hit < 0) obj->o_flags |= ISCURSED;

	when RING:
	    obj->o_ac = hit;
	    switch (obj->o_which) {
		when R_ADDSTR:
		case R_ADDWISDOM:
		case R_ADDINTEL:
		case R_PROTECT:
		case R_ADDHIT:
		case R_ADDDAM:
		case R_DIGEST:
		    if (hit > 1) obj->o_flags |= ISBLESSED;
		    else if (hit < 0) obj->o_flags |= ISCURSED;
	    }

	when STICK:
	    fix_stick(obj);

	    obj->o_charges = hit;

	when GOLD:
	    obj->o_type = GOLD;
	    obj->o_count = GOLDCALC;
	    obj->o_ac = 11;
    }
    return(item);
}

/*
 * pick an item out of a list of nitems possible magic items
 */
pick_one(magic, nitems)
register struct magic_item *magic;
int nitems;
{
    register struct magic_item *end;
    register int i;
    register struct magic_item *start;


    start = magic;
    for (end = &magic[nitems], i = rnd(1000); magic < end; magic++)
	if (i < magic->mi_prob)
	    break;
    if (magic == end)
    {
	if (wizard)
	{
	    msg("bad pick_one: %d from %d items", i, nitems);
	    for (magic = start; magic < end; magic++)
		msg("%s: %d%%", magic->mi_name, magic->mi_prob);
	}
	magic = start;
    }
    return magic - start;
}


/* blesscurse returns whether, according to the flag, the object is
 * blessed, cursed, or neither
 */

char *
blesscurse(flags)
int flags;
{
    if (flags & ISKNOW)  {
	if (flags & ISCURSED) return("cursed ");
	if (flags & ISBLESSED) return("blessed ");
	return("normal ");
    }
    return("");
}

/*
 * extras:
 *	Return the number of extra items to be created
 */
extras()
{
	register int i;

	i = rnd(100);
	if (i < 4)		/* 4% for 2 more */
	    return (2);

	else if (i < 11)	/* 7% for 1 more */
	    return (1);
	else			/* otherwise no more */
	    return (0);
}
