/* Tetrix by quentin */

#ifndef ATARIST

#include <signal.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#endif

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <curses.h>
#include "tet.h"

/********** Some global variable declarations ************/
int Type;		/* type specifies rotation, shape and color of blocks */
int Row;		/* Row of pivot point of block */
int Column;		/* Column of pivot point of block */
int Pause;		/* Time between movements this block */
int CurrentPause;	/* Time between movements per level */
int FallingDown;	/* True when space bar is pressed */
int Beep;
char Key;		/* holds last key polled */

#ifdef ATARIST

#define SCORE_FILE	"c:\\etc\\scores.tet"

cuserid(name)
char *name;
{
    register i, c;

    while (Cconis())
	Crawcin();
    mvaddstr(23,0,"Name? ");
    Cconws("\033e");

    while (1) {
	for (i=0; (c=Crawcin()) != '\r';)
	    if (c == 8) {
		if (i > 0) {
		    Cconws("\010 \010");
		    --i;
		}
	    }
	    else if (i < 9 && c >= ' ' && c <= '~')
		Cconout(name[i++] = c);
	if (i)
	    break;
	mvaddstr(23,6, "?\010");
    }
    name[i] = 0;
    
    Cconws("\033f");
    mvaddstr(23,0,"                   ");
}

mrand48()
{
    return Random();
}

#else
# ifndef SCORE_FILE
#  define SCORE_FILE	"/usr/share/games/TetScores"
# endif
# ifndef SCORE_LOCK
#  define SCORE_LOCK	"/var/spool/locks/TetScores"
# endif
#endif

char ScoreString[10];
struct ScoreStruct {
	char Name[10];
	long Score;	/* Grrr.... */
} High[10];
long ThisScore;
int HighsChanged;
		
#ifndef ATARIST

char *ttyname();
char combuf[2];
struct termios origtty, tty;

int game_uid;

#endif

char Board[BOARD_WIDE][BOARD_HIGH];
char Temp[BOARD_WIDE][BOARD_HIGH];	/* temp storage for TestRows */

/* debug flash to screen */
#define FLASHMSG(x)
/*
#define FLASHMSG(x)	      { mvaddstr(23,0,"                "); \
				mvaddstr(23,0,x); \
				refresh(); }
*/
#define UPSCORE(x)    { ThisScore += x; \
			sprintf((char *)ScoreString,"%-ld",ThisScore); \
			color(W_CHAR|IS_BOLD); \
			mvaddstr(1,46,ScoreString); }

#define NULL_KEY	'\0'
#define FALL_KEY	' '
#define RIGHT_KEY	'l'
#define LEFT_KEY	'j'
#define ROTATE_KEY	'k'
#define L_RIGHT_KEY	'f'		/* for south paws */
#define L_LEFT_KEY	's'
#define L_ROTATE_KEY	'd'
#define QUIT_KEY	'q'
#define BEEP_KEY	'b'
#define BOSS_KEY	'!'
#define	PAUSE_KEY	'\033'
#define PLAY_KEY	'p'
#define SCORE_KEY	'h'
#define MENU_KEY	'm'

/**************************************************MAIN*****/
main()
{
    Init();
    for ( ; ; ) {
	NewGame();
	Play();
	ScoreIt();
	DrawScore();
    }
}

/*************************************************************/
Init()
{
    register char *p;
    register int x, y, i, fd;

#ifndef ATARIST
    int timein;

    time(&timein);		/* get start time */
    srand48(timein);	/* start rand randomly */

    signal(SIGINT, SIG_IGN);
    /*
     * setup raw mode, no echo
     */
    tcgetattr(0, &origtty);
    tcflush(0, TCIOFLUSH);
    tty = origtty;
    tty.c_iflag &= ~(IGNCR|IUCLC|ICRNL|INLCR);
#ifdef NCURSES
    tty.c_oflag &= ~(ONLCR|OPOST|ONLCR|OCRNL|ONLRET|OLCUC);
#endif
    tty.c_lflag &= ~(ISIG|ICANON|XCASE|ECHO|ECHOE|ECHOK|ECHONL|ECHOPRT|ECHOCTL|ECHOKE);
    tty.c_cc[VMIN] = 1; tty.c_cc[VTIME] = 0;
    tcsetattr(0, TCSADRAIN, &tty);

    game_uid = geteuid();	/* get the game uid, then drop privileges */
    setuid(getuid());		/* until we actually need them */
#endif

    Beep=0;
    HighsChanged = 0;
    /* ScoreIt(); */
    initscr();
#ifdef NCURSES
    if (has_colors()) {
	start_color();
	init_pair(1, COLOR_GREEN, COLOR_BLACK);
	init_pair(2, COLOR_RED, COLOR_BLACK);
	init_pair(3, COLOR_CYAN, COLOR_BLACK);
	init_pair(4, COLOR_MAGENTA, COLOR_BLACK);
	init_pair(5, COLOR_BLUE, COLOR_BLACK);
	init_pair(6, COLOR_YELLOW, COLOR_BLACK);
	init_pair(7, COLOR_CYAN, COLOR_BLACK);
    }
    curs_set(0);
#endif

    /* initilialize board to spaces */
    for (x=0; x<BOARD_WIDE; x++) 
	for (y=0; y<BOARD_HIGH; y++) 
	    PUTCH(x,y,NO_CHAR);
    erase();
    DrawMenu();
    refresh();
}

#ifdef NCURSES
color(col)
{
    int color_code = COLOR_WHITE;
    int mode = A_STANDOUT;

    if (has_colors()) {
	if (col == 0 || col == NO_CHAR) {
	    attrset(A_NORMAL|COLOR_PAIR(0));
	    return;
	}
	switch (COLOR_OF(col)) {
	case G_CHAR:	color_code = 1;	break;
	case R_CHAR:	color_code = 2;	break;
	case T_CHAR:	color_code = 3;	break;
	case V_CHAR:	color_code = 4;	break;
	case B_CHAR:	color_code = 5;	break;
	case Y_CHAR:	color_code = 6;	break;
	default:
	case W_CHAR:	color_code = 7;	break;
	}
	switch (MODE_OF(col)) {
	case IS_NORMAL: mode = A_NORMAL;break;
	case IS_BOLD: mode = A_BOLD;	break;
	default: mode = A_STANDOUT;	break;
	}
	attron(mode|COLOR_PAIR(color_code));
    }
}

drawbox(y, x, dy, dx)
{
    int ix, iy;


    color(V_CHAR|IS_BOLD);

    for (ix=1; ix<dx-1; ix++) {
	mvaddch(y,      x+ix, ACS_HLINE);
	mvaddch(y+dy-1, x+ix, ACS_HLINE);
    }
    for (iy=1; iy<dy-1; iy++) {
	mvaddch(y+iy, x,      ACS_VLINE);
	mvaddch(y+iy, x+dx-1, ACS_VLINE);
    }
    mvaddch(y,      x,      ACS_ULCORNER);
    mvaddch(y+dy-1, x,      ACS_LLCORNER);
    mvaddch(y,      x+dx-1, ACS_URCORNER);
    mvaddch(y+dy-1, x+dx-1, ACS_LRCORNER);
    color(0);
}
#else
color(x) {}
#endif


#ifndef NCURSES
beep() { write(1, "\007", 1); }
#endif

/**************************************************************/
NewGame()
{
    register int x,y;

    CurrentPause=0;

    while (!CurrentPause) {
	switch (GetKey(0)) {
	    case BEEP_KEY:
		if (Beep = !Beep)
		    beep(); 
		break;
	    case SCORE_KEY:
		GetScore();
		DrawScore();
		break;
	    case MENU_KEY:
		DrawMenu();
		break;
	    case PAUSE_KEY:
		GetKey(0);
		break;
	    case BOSS_KEY:
		Boss();
		break;
	    case PLAY_KEY:
		CurrentPause=1000;
		break;
	    case QUIT_KEY:
		Leave();
	}
    }
    /* initilialize board to spaces */
    for (x=0; x<BOARD_WIDE; x++) 
	for (y=0; y<BOARD_HIGH; y++) 
	    PUTCH(x,y,NO_CHAR);
    ThisScore=0;
#ifdef NCURSES
    color(W_CHAR|IS_BOLD);
    mvaddstr(1,43, "  ......  ");
#else
    mvaddstr(1,42,"|  ......  |");
#endif
    UPSCORE(0);

}

/******************************************************************/
DrawMenu()
{
    register int y;
    erase(); 

    /* draw score border */
#ifdef NCURSES
    drawbox(0,42, 3, 12);
    color(W_CHAR|IS_BOLD);
    mvaddstr(1, 43, "  ......");
#else
    mvaddstr(0,42,".----------.");
    mvaddstr(1,42,"|  ......  |");
    mvaddstr(2,42,"`----------'");
#endif
    UPSCORE(0);

    /* draw menu */
#ifdef NCURSES
    drawbox(4, 35, 18, 29);
    color(W_CHAR|IS_BOLD);
    mvaddstr( 6,36,"      ..   Menu   ..     ");
    mvaddstr( 8,36,"   h     .... high scores");
    mvaddstr( 9,36,"   b     .... toggle beep");
    mvaddstr(10,36,"   p     .... play");
    mvaddstr(11,36,"   q     .... quit");
    mvaddstr(13,36," s or j  .... move left");
    mvaddstr(14,36," d or k  .... rotate piece");
    mvaddstr(15,36," f or l  .... move right");
    mvaddstr(16,36,"  spc    .... fall piece");
    mvaddstr(17,36,"  esc    .... pause");
    color(0);
#else
    mvaddstr( 4,35,".---------------------------.");
    mvaddstr( 5,35,"|                           |");
    mvaddstr( 6,35,"|      ..   Menu   ..       |");
    mvaddstr( 7,35,"|                           |");
    mvaddstr( 8,35,"|   h     .... high scores  |");
    mvaddstr( 9,35,"|   b     .... toggle beep  |");
    mvaddstr(10,35,"|   p     .... play         |");
    mvaddstr(11,35,"|   q     .... quit         |");
    mvaddstr(12,35,"|                           |");
    mvaddstr(13,35,"| s or j  .... move left    |");
    mvaddstr(14,35,"| d or k  .... rotate piece |");
    mvaddstr(15,35,"| f or l  .... move right   |");
    mvaddstr(16,35,"|  spc    .... fall piece   |");
    mvaddstr(17,35,"|  esc    .... pause        |");
    mvaddstr(18,35,"|                           |");
    mvaddstr(19,35,"| LATEST: allow concurrent  |");
    mvaddstr(20,35,"|    high score setting     |");
    mvaddstr(21,35,"`---------------------------'");
#endif

    /* draw game border */
#ifdef NCURSES
    drawbox(0, 14, 22, 12);
#else
    mvaddstr(0,14, ".----------.");
    mvaddstr(21,14,"`----------'");
    for (y=1; y<21; y++)
	mvaddstr(y,14,"|          |");
#endif

    /* display the title */
    color(R_CHAR);
    mvaddstr(3,17,"TETRIX");
    refresh();
}

/**************************************************************/
Play()
{
    while ((Key != QUIT_KEY) && NewPiece()) {
	FallingDown = 0;
	do {
	    /* do until we can't Advance the piece */
	    if (FallingDown)
		Pause = 0;
	    else
		Pause = CurrentPause;

	    do {
		switch (GetKey(&Pause)) {
		case PAUSE_KEY:
		    Wait("press a key");
		    break;
		case BOSS_KEY:
		    Boss();
		    Wait("press a key");
		    break;
		case QUIT_KEY:
		    CurrentPause = 0;
		case FALL_KEY:
		    FallingDown = 1;
		    UPSCORE(20-Row);
		    Pause = 0;
		    break;
		case RIGHT_KEY:
		case L_RIGHT_KEY:
		    MoveRight();
		    break;
		case 'h':
		case LEFT_KEY:
		case L_LEFT_KEY:
		    MoveLeft();
		    break;
		case ROTATE_KEY:
		case L_ROTATE_KEY:
		    Rotate();
		    break;
		case NULL_KEY:
		    break;
		default:
		    if (Beep)
			beep();
		}
	    } while (Pause > 0);
	} while (AdvancePiece());
	UPSCORE(5);
	TestRows();
    }
}

/*
 * lockscore() -- locks the scorefile, or waits until it can be locked
 */
lockscore()
{
    int fd, i;
    int status = 0;
    char pid[20];

    for (i=0; i<5; i++) {
	if (fd = open(SCORE_LOCK, O_CREAT|O_EXCL, 0666)) {
	    sprintf(pid, "%010d\n", getpid());
	    write(fd, pid, strlen(pid));
	    close(fd);
	    status = 1;
	    break;
	}
	if ((fd = open(SCORE_LOCK, O_RDONLY)) < 0) {
	    /* lockfile exists, can't open it for reading! */
	    status = 0;
	    break;
	}
	read(fd, pid, 10);
	if (kill(atoi(pid), 0) != 0)
	    /* stale lock -- kill it and try again */
	    unlink(SCORE_LOCK);
	sleep(1);
	errno = EBUSY;
    }
    return status;
}

/*
 * unlockscore() drops the scorefile so that other people can get to it
 */
unlockscore()
{
    unlink(SCORE_LOCK);
}


GetScore()
{
    int fd;
    int i;

    if ((fd=open(SCORE_FILE, O_RDONLY)) != -1) {
	read(fd,High,sizeof(High));
	close(fd);
    }
    else {
	for(i=0; i<10; i++)
	    High[i].Score = 0;
	for(i=0; i<10; i++)
	    strncpy(High[i].Name, "         ",10);
    }
}


/*********************************************************************/
ScoreIt()
{
    register int fd,i,j;

    putchar(7);

    if (!lockscore()) {
	mvprintw(22,0,"Couldn't lock scorefile (%s)", strerror(errno));
	return;
    }

    GetScore();

    for (i=0; i<10; i++)		/* place this guy */
	if (High[i].Score <= ThisScore)
	    break;

    if (i < 10 ) {			/* insert this score */
	HighsChanged = 1;
	for (j=9; j>i; j--) {		/* move down others */
	    if (High[j-1].Score) {
		High[j].Score = High[j-1].Score;
		strncpy(High[j].Name,High[j-1].Name,10);
	    }
	}
	cuserid((char *) High[i].Name);
	High[i].Score = ThisScore;
    }

    if (HighsChanged) {
	setuid(game_uid);
	if ((fd=open(SCORE_FILE, O_WRONLY|O_CREAT, 0644)) != -1) {
	    write(fd,High,sizeof(High));
	    close(fd);
	}
	else
	    mvprintw(22,0,"Couldn't save scorefile (%s)", strerror(errno));
	setuid(getuid());
    }
    unlockscore();
}

/***********************************************************************/
DrawScore()
{
    register int j;
    char lbuf[80];

#ifdef NCURSES
    color(W_CHAR|IS_NORMAL);
    mvaddstr( 5,36,"     Hit 'm' for menu      ");
    mvaddstr( 6,36,"                           ");
    mvaddstr( 7,36,"        HIGH SCORES        ");
    mvaddstr( 8,36," 1.                        ");
    mvaddstr( 9,36," 2.                        ");
    mvaddstr(10,36," 3.                        ");
    mvaddstr(11,36," 4.                        ");
    mvaddstr(12,36," 5.                        ");
    mvaddstr(13,36," 6.                        ");
    mvaddstr(14,36," 7.                        ");
    mvaddstr(15,36," 8.                        ");
    mvaddstr(16,36," 9.                        ");
    mvaddstr(17,36,"10.                        ");
    mvaddstr(18,36,"                           ");
    mvaddstr(19,36,"                           ");
    mvaddstr(20,36,"                           ");
    color(0);
#else
    mvaddstr( 5,35,"|     Hit 'm' for menu      |");
    mvaddstr( 6,35,"|                           |");
    mvaddstr( 7,35,"|        HIGH SCORES        |");
    mvaddstr( 8,35,"| 1.                        |");
    mvaddstr( 9,35,"| 2.                        |");
    mvaddstr(10,35,"| 3.                        |");
    mvaddstr(11,35,"| 4.                        |");
    mvaddstr(12,35,"| 5.                        |");
    mvaddstr(13,35,"| 6.                        |");
    mvaddstr(14,35,"| 7.                        |");
    mvaddstr(15,35,"| 8.                        |");
    mvaddstr(16,35,"| 9.                        |");
    mvaddstr(17,35,"|10.                        |");
    mvaddstr(18,35,"|                           |");
    mvaddstr(19,35,"|                           |");
    mvaddstr(20,35,"|                           |");
#endif

    for (j=0; j<10; j++) {
	if (High[j].Score) {
	    sprintf(lbuf,"%-s",(char *)High[j].Name); mvaddstr(j+8,41,lbuf);
	    sprintf(lbuf,"%ld",High[j].Score); mvaddstr(j+8,54,lbuf);
	}
    }
    refresh();

}

/*********************************************************************/
Boss()
{
    register int x,y;
    char *shell;
    pid_t pid;

#ifdef ATARIST
    erase();
    system("tsh");	/* fork off a shell... */
    initscr();
#else

    if ((shell = getenv("SHELL")) == (char*)0)
	shell = "/bin/sh";
    if ((pid = fork()) < 0)
	Wait("Oops");
    else if (pid == 0) {	/* child */
	clear();
	refresh();
	endwin();
	tcsetattr(0, TCSADRAIN, &origtty);
	execl(shell, shell, 0);
    }
    else {			/* parent */
	int st;
	waitpid(pid, &st, 0);
	tcsetattr(0, TCSADRAIN, &tty);
	clear();
	touchwin(stdscr);
    }
#endif
    DrawMenu();
    /* restore board */
    for (x=0; x<BOARD_WIDE; x++) 
	for (y=0; y<BOARD_HIGH; y++) 
	    PUTCH(x,y,Board[x][y]);
    refresh();
}

/*********************************************************************/
GetKey(int *timeout)
{
/*    fflush(stdout); */
#ifdef ATARIST
    Key = NULL_KEY;
    if (!Cconis())
	return;
    Key = Crawcin();
#else
    fd_set rd;
    struct timeval timer;

    Key = NULL_KEY;
    if (timeout) {
	FD_ZERO(&rd);
	FD_SET(0, &rd);

	timer.tv_sec = (*timeout) / 1000;
	timer.tv_usec = ((*timeout) % 1000)*1000;

	select(1, &rd, 0, 0, &timer);

	if (FD_ISSET(0, &rd)) {
	    read(0, combuf, 1);
	    Key = (*combuf & 0177);
	}
	*timeout = (timer.tv_sec*1000) + ((timer.tv_usec+500) / 1000);
    }
    else {
	read(0, combuf, 1);
	Key = (*combuf & 0177);
    }
    return Key;
#endif
}

/************************************************************************/
/* Could be a macro for speed but cpp runs out of tree space in CanMove */
IS_FREE(x,y)
int x,y;
{
    if ((y < 0) || (y >= BOARD_HIGH) || (x < 0) || (x >= BOARD_WIDE))
	return(0);
    if (Board[x][y] != NO_CHAR)
	return(0);
    else
	return(1);
}

/*********************************************************************/
TestRows()
{
    register int x,y,tempy,fullrow;
    int marked[BOARD_HIGH];

    for (y=0; y<BOARD_HIGH; y++) {
	marked[y] = 0;
	for (x=0; x<BOARD_WIDE; x++)
	    Temp[x][y] = NO_CHAR;
    }

    /* main loop to traverse Board, looking for fullrows */
    /* as it looks, it copies non full ones over to Temp */
    tempy=BOARD_HIGH-1;
    for (y=BOARD_HIGH-1; y>=0; y--) {
	fullrow = 1;
	for (x=0; x<BOARD_WIDE; x++)		/* check for any holes at all */
	    if (IS_FREE(x,y)) {
		fullrow = 0;
		break;
	    }
	if (fullrow) {
	    marked[y]++;
	    CurrentPause -= 2;			/* speed up the game */
	}
	else {
	    for (x=0; x<BOARD_WIDE; x++)
		Temp[x][tempy] = Board[x][y];
	    tempy--;
	}
    }

    /* flash the rows that will die */
    for (tempy=1; tempy<5; tempy++)
	for (y=BOARD_HIGH-1; y>=0; y--) 
	    if (marked[y]) {	
		UPSCORE(30-y);
		for (x=0; x<BOARD_WIDE; x++)	
		    PUTCH(x,y,BRITE_CHAR);
		refresh();
		for (x=0; x<BOARD_WIDE; x++)	
		    PUTCH(x,y,NO_CHAR);
		refresh();
	    }

    /* Move temp back to Board */
    for (y=BOARD_HIGH-1; y>=0; y--) {
	for (x=0; x<BOARD_WIDE; x++)
	    PUTCH(x,y,Temp[x][y]);
	refresh();
    }
}

Wait(msg)
char *msg;
{
    mvaddstr(22, 20, msg);
    GetKey(0);
    mvprintw(22, 20, "%*.*s", strlen(msg), strlen(msg), "");
}

/***********************************************************/
Leave()
{
    erase();
    mvaddstr(22,48,"Ciao.\n");
    mvaddstr(23,0,"");
    refresh();
    endwin();
#ifdef ATARIST
    Cconws("\033e");
#else
    tcsetattr(0, TCSADRAIN, &origtty);
#endif
    exit(0);
}
