#
# functions used to build mastodon packages
# 
# exedir [VARIABLE] [package-name]
#	locals: _pwd=`pwd`
#		_up=`cd ../..;pwd`
# 	puts the path to place the compiled package into VARIABLE.  If
# 	package-name is not defined, it defaults to
#		`basename $_pwd | tr A-Z a-z | sed -e 's/-[0-9].*$//'`
#	if VARIABLE is not defined, it defaults to
#		`basename $_up` | tr a-z A-Z`
#
# supdir AREA [VARIABLE] [package-name]
#	like exedir, except it builds a supplemental package in AREA.
#	package-name defaults as in exedir, VARIABLE defaults to AREA.
#
# setmanpermissions $AREA
#	sets permissions of manpages
#

supdir() {
    _pwd=`pwd`
    _up=`cd ../../..;pwd`
    _area=`cd $_up/$1;pwd`

    if [ "$#" -lt 1 ]; then
	echo "missing AREA to supdir in `pwd`/$0"
	exit 1
    fi

    _variable=${2:-`echo $1 | tr a-z A-Z`}
    _packagename=${3:-`basename $_pwd | tr A-Z a-z | sed -e 's/-[0-9].*$//'`}

    rm -rf $_area/$_packagename
    mkdir -p $_area/$_packagename

    eval $_variable=$_area/$_packagename

    __packages="$__packages $_area/$_packagename"
}

exedir() {
    _bin=`cd ../..;pwd`
    supdir `basename $_bin` $1 $2
    _basepkg=$_area/$_packagename

    touch $_bin/Build
    test -f .gpl && ln -s `pwd` $_basepkg/.source
}

setmanpermissions() {
    if [ $# -eq 0 ]; then
	for x in $__packages;do
	    if [ -d $x/usr/man ]; then
		echo "setting permissions on $x/usr/man"
		chown -R man.man $x/usr/man
		find $x/usr/man -type f -print |xargs chmod 444
	    fi
	    if [ -d $x/usr/X11R6/man ]; then
		echo "setting permissions on $x/usr/X11R6/man"
		chown -R man.man $x/usr/X11R6/man
		find $x/usr/X11R6/man -type f -print |xargs chmod 444
	    fi
	    if [ -d $x/usr/doc ]; then
		echo "setting permissions on $x/usr/doc"
		chown -R man.man $x/usr/doc
		find $x/usr/doc -type f -print | xargs chmod 444
	    fi
	    if [ -d $x/usr/info ]; then
		echo "setting permissions on $x/usr/info"
		chown -R man.man $x/usr/info
		find $x/usr/info -type f -print | xargs chmod 444
	    fi
	done
    fi
}

install_manpages() {
    if [ "$_basepkg" ]; then
	__orig_pwd=`pwd`
	cd "${1:-.}"
	for x in *.[1-8];do
	    if [ "$x" != '*.[1-8]' ]; then
		EXT=`echo $x | sed -ne 's/^.*\(.\)$/\1/p'`
		_dir=$_basepkg/usr/man/man$EXT
		test -d $_dir || mkdir -p $_dir
		cp $x $_dir
	    fi
	done
	cd $__orig_pwd
    fi
}

checkdirectories() {
    for x in $__packages; do
	if [ -d $x/usr/X11R6/lib/X11 ]; then
	    mkdir -p $x/var/X11R6
	    mv $x/usr/X11R6/lib/X11 $x/var/X11R6/lib
	    ln -s /var/X11R6/lib $x/usr/X11R6/lib/X11
	    echo "Fixed /usr/X11R6/lib/X11 in $x"
	fi
	if [ -d $x/var/spool/at/jobs ]; then
	    chown bin.bin $x/var/spool/at/jobs
	    for z in $x/var/spool/at/jobs/*; do
		if [ -f $z ]; then
		    PERM=`grep '^# atrun' $z`
		    eval `echo "$PERM" | sed -ne 's/^.*\(uid=[0-9]*\).*$/\1/p'`
		    eval `echo "$PERM" | sed -ne 's/^.*\(gid=[0-9]*\).*$/\1/p'`
		    chown $uid.$gid $z
		    echo "chown $uid.$gid $z"
		fi
	    done
	fi
	if [ -d $x/var/spool/at/spool ]; then
	    chown bin.bin $x/var/spool/at/spool
	fi
    done
}

unset __packages
trap "checkdirectories;setmanpermissions" 0

SYSCC="cc -bia32-linuxaout -m386"
MASTODON=/Mastodon
